/*
 * file StpExceptionStpReasonCodeVisitor.java
 *
 * IBM Confidential
 * OCO Source Materials
 * Rational ClearCase
 * (C) Copyright IBM Corp. 2010.  All rights reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.rational.wvcm.stpex.annotation.processing;

import static com.ibm.rational.wvcm.stpex.annotation.processing.ListClassesVisitor.INDENT;
import static com.ibm.rational.wvcm.stpex.annotation.processing.ListClassesVisitor.createFile;
import static com.ibm.rational.wvcm.stpex.annotation.processing.ListClassesVisitor.ensureUniqueAndAddIfSo;
import static com.ibm.rational.wvcm.stpex.annotation.processing.ListClassesVisitor.getRpcEnumValues;
import static com.ibm.rational.wvcm.stpex.annotation.processing.ListClassesVisitor.writeCommonHeader;

import java.io.PrintWriter;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeSet;

import com.sun.mirror.apt.AnnotationProcessorEnvironment;
import com.sun.mirror.apt.Filer;
import com.sun.mirror.declaration.EnumConstantDeclaration;
import com.sun.mirror.util.SimpleDeclarationVisitor;

public class StpExceptionStpReasonCodeVisitor extends SimpleDeclarationVisitor implements StpVisitor{
    private final Map<Integer, TagInfo> m_enums = new HashMap<Integer, TagInfo>();
    private final AnnotationProcessorEnvironment m_env;
    
    private static final String g_hxxFileName = "stp_reason_codes.hxx";
    private static final String g_javaFileName = "STPReasonCodeMap.java";
    
    private PrintWriter m_hxxOut;
    private PrintWriter m_javaOut;
    
    public StpExceptionStpReasonCodeVisitor(AnnotationProcessorEnvironment env)
    {
        //System.out.println("Creating StpExceptionStpReasonCodeVisitor");
        m_env = env;
    }

    public void begin() {
        //System.out.println("StpExceptionStpReasonCodeVisitor begin");
        Filer filer = m_env.getFiler();
        writeHxxBegin(filer);
        writeJavaBegin(filer);
    }

    public void end() {
        TreeSet<Integer> keys = new TreeSet<Integer>(m_enums.keySet());
        writeHxxEnd(keys);
        writeJavaEnd(keys);
        //System.out.println("StpExceptionStpReasonCodeVisitor end");
    }
    
    public void writeHxxBegin(Filer filer)
    {
        m_hxxOut = createFile(filer, g_hxxFileName);
        writeCommonHeader(m_hxxOut, this);
    }
    
    public void writeHxxEnd(TreeSet<Integer> keys)
    {
        boolean first = true;
        for (Integer id : keys) {
            TagInfo info = m_enums.get(id);
            if (!first) {
                m_hxxOut.println(",");
            }
            m_hxxOut.print(INDENT + info.fieldName + " = " + id);
            if (first) first = false;
        }
        m_hxxOut.println("");
        
        m_hxxOut.flush();
        m_hxxOut.close();        
    }

    private void writeJavaBegin(Filer filer)
    {
        m_javaOut = createFile(filer, g_javaFileName);

        writeCommonHeader(m_javaOut, this);

        m_javaOut.println("package com.ibm.rational.stp.server.rpc;");
        m_javaOut.println("");
        m_javaOut.println("import java.util.Map;");
        m_javaOut.println("import java.util.HashMap;");
        m_javaOut.println("import com.ibm.rational.wvcm.stp.StpException;");
        m_javaOut.println("");
        m_javaOut.println("/**");
        m_javaOut.println(" * Mapping services from/to StpException.StpReasonCodes to/from");
        m_javaOut.println(" * RPC enumerated reason code names.");
        m_javaOut.println(" */");
        m_javaOut.println("public class STPReasonCodeMap");
        m_javaOut.println("{");
        m_javaOut.println("    /**");
        m_javaOut.println("     * Get the StpReasonCode associated with RPC error code.");
        m_javaOut.println("     *");
        m_javaOut.println("     * @param code IN: RPC integer value error code");
        m_javaOut.println("     *");
        m_javaOut.println("     * @return StpReasonCode associated with RPC error code");
        m_javaOut.println("     */");
        m_javaOut.println("    public static StpException.StpReasonCode");
        m_javaOut.println("    getSTPReasonCode(int code) { return g_reasonCodes.get(code); }");
        m_javaOut.println("");
        m_javaOut.println("    private static final Map<Integer, StpException.StpReasonCode> g_reasonCodes = new HashMap<Integer, StpException.StpReasonCode>();");
        m_javaOut.println("");
        m_javaOut.println("    static {");
        m_javaOut.println("        ");
    }

    private void writeJavaEnd(TreeSet<Integer> keys)
    {
        for (Integer id : keys) {
            TagInfo info = m_enums.get(id);
            m_javaOut.println("        g_reasonCodes.put(" + info.id + ", StpException.StpReasonCode." + info.fieldName + ");"); 
        }

        m_javaOut.println("    }");
        m_javaOut.println("}");

        m_javaOut.flush();
        m_javaOut.close();
    }
    
    @Override
    public void visitEnumConstantDeclaration(EnumConstantDeclaration ecd) {

        TagInfo info = getRpcEnumValues(ecd.getAnnotationMirrors());
        if (info != null) {
            info.fieldName = ecd.getSimpleName();
            ensureUniqueAndAddIfSo(m_enums, info);
        }
    }
}
